/*
 * --- file timer.h ---
 *
 * This is the file header for the timer module. This module is a layer to setup the four
 * timers that the LPC1769 has (TIMER0/1/2/3). This code is based on a Code Red Technologies
 * example, but all the prototypes and functions are being rewrited
 * @Author: Carles Saura
 */


#ifndef __TIMER_H 
#define __TIMER_H

#include <stdint.h>

extern volatile uint32_t timer0_counter;
extern volatile uint32_t timer1_counter;
extern volatile uint32_t timer2_counter;
extern volatile uint32_t timer3_counter;
/*-----------------------------------------------------------*/

/*
 * This function power on the appropriate timer
 * @precondition timer_num must to be a value between 0 and 3, other values will be
 * interpreted as a 3 value.
 *
 * @param timer_num		The number of the timer
 */
void timer_power( uint8_t timer_num );



/*
 * This function resets the appropriate timer
 * @precondition timer_num must to be a value between 0 and 3, other values will be
 * interpreted as a 3 value.
 *
 * @param timer_num		The number of the timer
 */
void timer_reset( uint8_t timer_num );

/*
 * This function set the preescaler on the appropriate timer
 * @precondition timer_num must to be a value between 0 and 3 as described on table 42,
 * page 56 in the UM10360 manual, other values will be interpreted as a 3 value.
 * @precondition timer setting must be a value between 0x00 and 0x03, if this condition
 * is not fulfilled the behavior can be unpredictable and probably a HardFaultHandler will
 * be called.
 *
 * @param timer_num		The number of the timer
 * @param timer_setting	The setting value, one of { 0x00, 0x01, 0x02, 0x03 }
 */
void timer_prescaler(uint8_t timer_num, uint8_t timer_setting);

/*
 * This initialises appropriate timer setting up the registers on it, establishing the time
 * interval, the type of behavior on interrupt ( as Interrupt + reset ), resets the counter
 * register and enables the interrupt.
 *
 * @precondition timer_num must to be a value between 0 and 3, other values will be
 * interpreted as a 3 value.
 *
 * @param timer_num		The number of the timer
 * @param timerInterval	The time interval
 */
void timer_init( uint8_t timer_num, uint32_t timerInterval );

/*
 * This function enable on the appropriate timer
 * @precondition timer_num must to be a value between 0 and 3, other values will be
 * interpreted as a 3 value.
 *
 * @param timer_num		The number of the timer
 */
void timer_enable( uint8_t timer_num );
/*
 * This function disables appropriate timer
 * @precondition timer_num must to be a value between 0 and 3, other values will be
 * interpreted as a 3 value.
 *
 * @param timer_num		The number of the timer
 */
void timer_disable( uint8_t timer_num );

#endif /* end __TIMER_H */

