/*
 * --- file midi.c ---
 *
 * This is the file implementation for the midi module
 * @Author: Carles Saura
 */
#include "midi.h"
/* CLib includes */
#include <stdint.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
/* LPC includes */
#ifndef LPCXPRESSO1769
	#include "LPC17xx.h"
#endif
#ifndef INC_FREERTOS_H
	#include "FreeRTOS.h"
#endif
#ifndef TASK_H
	#include "task.h"
#endif
#ifndef SEMAPHORE_H
	#include "semphr.h"
#endif
#ifndef __UART_H
	#include "uart.h"
#endif
extern uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
extern uint8_t UART0Buffer, UART1Buffer, UART2Buffer, UART3Buffer;
volatile uint32_t *midi_lpc_uart_ier;
volatile uint32_t *midi_uart_count;
volatile uint32_t *midi_uart_countIn;
uint8_t *midi_uart_buffer;
uint8_t midi_uart_port;
xSemaphoreHandle *midi_semUART;
xSemaphoreHandle midi_semUART0 = NULL;
xSemaphoreHandle midi_semUART1 = NULL;
xSemaphoreHandle midi_semUART2 = NULL;
xSemaphoreHandle midi_semUART3 = NULL;
xSemaphoreHandle midi_trans_sem = NULL;
uint8_t midi_trans_mode = 0;
uint8_t midi_trans_id = 255;
portTickType midiPreviousWakeTime = -1;

/*
 * Simple function to setup the pointer configuration.
 * @precondition 	port must be a value between 0 and 3, otherwise must be considered as 3
 *
 * @param port  	Port to initialize the connection
 */
void midi_confUART ( uint8_t port ){
	midi_uart_port = port;
	switch(port){
	case 0: midi_lpc_uart_ier = &LPC_UART0->IER;
		midi_uart_count = &UART0Count;
		midi_uart_buffer = &UART0Buffer;
		midi_semUART = &midi_semUART0;
		break;
	case 1: midi_lpc_uart_ier = &LPC_UART1->IER;
		midi_uart_count = &UART1Count;
		midi_uart_buffer = &UART1Buffer;
		midi_semUART = &midi_semUART1;
		break;
	case 2: midi_lpc_uart_ier = &LPC_UART2->IER;
		midi_uart_count = &UART2Count;
		midi_uart_buffer = &UART2Buffer;
		midi_semUART = &midi_semUART2;
		break;
	default:
	case 3: midi_lpc_uart_ier = &LPC_UART3->IER;
		midi_uart_count = &UART3Count;
		midi_uart_buffer = &UART3Buffer;
		midi_semUART = &midi_semUART3;
		break;
	}
}

uint8_t midi_startUART( uint8_t  port, uint32_t baudrate){
	/* This is to prevent re-write configuration and initiate the semphr */
	if( *midi_semUART == NULL){
		/* Initialize UART */
		midi_confUART( port );
		UARTInit(port, baudrate);
		/* Enable interrupts on UART RX */
		//*midi_lpc_uart_ier = IER_THRE | IER_RLS | IER_RBR; // enable interrupt RX UART

		vSemaphoreCreateBinary( *midi_semUART );
		midiPreviousWakeTime = xTaskGetTickCount();
		//if( midi_trans_sem == NULL) vSemaphoreCreateBinary( midi_trans_sem );
		return 1;
	}
	else{
		/* Setting global variables */
		midi_confUART( port );
		return 0;
	}
}

void midi_stop( uint8_t port){
	midi_confUART( port );
	*midi_semUART = NULL;
}

/*
 * Simple function to write to the UART.
 * @precondition 	str is a valid pointer
 * @precondition 	if port contains a non pre intitialized port, baudrate will be initialized at default value (9600)
 *
 * @param port  	Port to initialize the connection
 * @param str   	Pointer to destination buffer
 * @param format    A string with the formated text
 */
void midi_sendUART( uint8_t port, uint8_t* bytePointer, int bufsz ){

	if ( port != midi_uart_port)
		midi_confUART(port);

	if ( *midi_semUART != NULL ){
		if (bufsz > BUFSIZE) bufsz = BUFSIZE;			/* This is to prevent buffer overflow */

		while ( xSemaphoreTake( *midi_semUART, ( portTickType ) 10 ) != pdTRUE ){}			/*  Wait and protect the function from other invocations */

		UARTSend(midi_uart_port, bytePointer , bufsz );
		*midi_uart_count = 0;
		/* Give the Semphr and enable the function to others invocations */
		xSemaphoreGive( *midi_semUART );
	}
}

void midi_print( uint8_t port, uint8_t trans_id, char* str, const char* format, ... ){
	//int bufsz;
	va_list listPointer;

	/* This is used to transmit the variable parameters */
	va_start( listPointer, format );
	/* Check if transaction mode is ON */
	if ( midi_trans_mode != 0 ){
		if ( trans_id != midi_trans_id){			/* If this invocation doesn't belongs from the transaction... */
			while ( xSemaphoreTake( midi_trans_sem, ( portTickType ) 30 ) != pdTRUE ){}			/*  Wait until transaction finish */
			xSemaphoreGive( midi_trans_sem );
		}
	}
	//bufsz = vsprintf(str, format, listPointer);
	va_end( listPointer );
	//midi_sendUART( port, str, bufsz);
}

void midi_vsprint( uint8_t port, uint8_t trans_id, char* str, const char* format, va_list listPointer ){
	//int bufsz;
	/* Check if transaction mode is ON */
	if ( midi_trans_mode != 0 ){
		if ( trans_id != midi_trans_id){			/* If this invocation doesn't belongs from the transaction... */
			while ( xSemaphoreTake( midi_trans_sem, ( portTickType ) 30 ) != pdTRUE ){}			/*  Wait until transaction finish */
			xSemaphoreGive( midi_trans_sem );
		}
	}
	//bufsz = vsprintf(str, format, listPointer);
	//midi_send( port, str, bufsz);
}


uint32_t midi_readUART( uint8_t port, uint8_t* dest ){
	uint32_t oldCount, newCount;

	if ( port != midi_uart_port)
		midi_confUART( port );

	if( *midi_uart_count != 0 ){
		newCount = *midi_uart_count;
		do{
			oldCount = newCount;
			vTaskDelayUntil( &midiPreviousWakeTime, 1/portTICK_RATE_MS );
			newCount = *midi_uart_count;
		}while( oldCount != newCount ); // wait a little time while we have new chars
		memcpy( dest, midi_uart_buffer, newCount );
		*midi_uart_count = 0; // clean buff

		return newCount;
	}
	else return 0;
}

uint8_t midi_transactionOn(){
	if ( midi_trans_mode == 0 ){
		if ( midi_trans_sem == NULL )
			vSemaphoreCreateBinary( midi_trans_sem );
		midi_trans_mode = 1;
		xSemaphoreTake( midi_trans_sem, ( portTickType ) 30 );
		midi_trans_id--;
		if ( midi_trans_id < 100)
			midi_trans_id = 255;
		return midi_trans_id;
	}
	else return 0;
}

void midi_transactionOff(){
	if ( midi_trans_sem != NULL )
		xSemaphoreGive( midi_trans_sem );
	midi_trans_mode = 0;
}

/*-----------------------------------------------------------*/
