/*
 * --- file pushButton.c ---
 *
 * This is the file implementation for the push button connected to the LPC1769
 * @Author: Carles Saura
 */
/* LPC includes */
#ifndef LPCXPRESSO1769
	#include "LPC17xx.h"
#endif
#ifndef INC_FREERTOS_H
	#include "FreeRTOS.h"
#endif
/* Utils includes */
#ifndef __CUSTOM_UTILS_H
	#include "utils.h"
#endif
#ifndef TASK_H
	#include "task.h"
#endif
#ifndef QUEUE_H
	#include "queue.h"
#endif
#ifndef SEMAPHORE_H
	#include "semphr.h"
#endif
#ifndef __TIMER_H
	#include "timer.h"
#endif
/* Debouncing delay value (in ms)*/
#define _PUSHBUTTON_DEBOUNCING_DELAY (50000) // ( ~20ms )
/* The queue used to save values. */
static xQueueHandle pushQueue = NULL;
/* The timer mark to delay */
portTickType pushButton_pxPreWT = -1;
/* This is used to know the kind of the interruption ( raising 0, falling 1 ) */
const unsigned long raiseFailling[ 2 ] = { 0UL,  100UL };
/* This is to know the port and the pin in the timer callback */
uint8_t pushButton_port = 0; // This default values are set as the first tests
uint8_t pushbutton_pin = 6; // developing this module, nostalgic reasons

/*-----------------------------------------------------------*/
/* Prototypes  */

/*
 * This private function sets the port and pin to know on timer call back where to check
 * @precondition port and pin must be valid values.
 *
 * @param port		The port number
 * @param pin		The pin number
 */
void pushButton_setPortPin( uint8_t port, uint8_t pin );

/*
 * This private function gets the port and pin set in pushButton_setPortPin().
 *
 * @return 		The port number
 */
uint8_t pushButton_getPort();

/*
 * This private function gets the pin and pin set in pushButton_setPortPin().
 *
 * @return  	The pin number
 */
uint8_t pushButton_getPin();

/*-----------------------------------------------------------*/
void pushButton_startInt( uint8_t  port, uint8_t pin){
	/* Create the queue. */
	pushQueue = xQueueCreate( 1, sizeof( unsigned long ) );
	/* Initialize pxPreviousTime */
	pushButton_pxPreWT = xTaskGetTickCount();
	/* Power up timer */
	timer_power ( 3 );
	/* Set prescaler to CLCK/4 */
	timer_prescaler( 3, 0x00 );
	/* Initialice timer */
	timer_init (  3,  _PUSHBUTTON_DEBOUNCING_DELAY );

	switch(port){
	case 2:
		LPC_GPIOINT->IO2IntEnR |= (1 << pin);
		LPC_GPIOINT->IO2IntEnF |= (1 << pin);
		break;
	default:
	case 0:
		LPC_GPIOINT->IO0IntEnR |= (1 << pin);
		LPC_GPIOINT->IO0IntEnF |= (1 << pin);
		break;
	}
	NVIC_EnableIRQ(EINT3_IRQn);
	/* Powering timer 0 to debounce */
	//LPC_SC->PCONP |= 1 << 1;
}

void pushButton_setPortPin( uint8_t port, uint8_t pin ){
	pushButton_port = port; // This default values are set as the first tests
	pushbutton_pin = pin;
}

uint8_t pushButton_getPort(){
	return pushButton_port;
}

uint8_t pushButton_getPin(){
	return pushbutton_pin;
}

void pushButton_raiseInt( uint8_t  port, uint8_t pin){
	if ( xQueueSendFromISR( pushQueue, &raiseFailling[ 0 ], 0 ) == pdTRUE){
		pushButton_setPortPin( port, pin );
		timer_reset( 3 );
		timer_enable( 3 );
	}
}


void pushButton_fallingInt( uint8_t  port, uint8_t pin){
	if ( xQueueSendFromISR( pushQueue, &raiseFailling[ 1 ], 0 ) == pdTRUE){
		pushButton_setPortPin( port, pin );
		timer_reset( 3 );
		timer_enable( 3 );
	}
}
uint8_t pushButton_timerCallback( unsigned long *toSend, unsigned long *toSet ){
	uint8_t pin;
	unsigned long receivedValue;

	pin = pushButton_getPin();
	if ( xQueueReceiveFromISR( pushQueue, &receivedValue, 0 ) == pdTRUE){
		if ( receivedValue == 100UL )
			if (!((LPC_GPIO0->FIOPIN & ( 1 << pin )) >> pin )){ 	/* It's stable */
				*toSend = 400UL;
				*toSet = pin;
				return pdTRUE;
			}
			else {
				return 0;
			}
		else //result = (!((LPC_GPIO0->FIOPIN & ( 1 << pin )) >> pin )) ? pdTrue : 0;
			if ((LPC_GPIO0->FIOPIN & ( 1 << pin )) >> pin ){	/* It's stable */
				*toSend = 500UL;
				*toSet = pin;
				return pdTRUE;
			}
			else {
				return 0;
			}
	}
	else return 0;
}
/*-----------------------------------------------------------*/
