
/*
 * --- file main.c ---
 *
 * This is the file part contains the code to implement a simple midi controller device.
 * Uses queues, timers, interruptions, and other setup related to the GPIO and ADC peripherals.
 *
 * @Author: Carles Saura
 */

/* Kernel includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "LPC17xx.h"
#ifndef __UART_H
	#include "uart.h"
#endif
#ifndef __TIMER_H
	#include "timer.h"
#endif
#include "adc.h"

/* Priorities at which the tasks are created. */
#define mainQUEUE_RECEIVE_TASK_PRIORITY		( tskIDLE_PRIORITY + 2 )
#define	mainQUEUE_SEND_TASK_PRIORITY		( tskIDLE_PRIORITY + 1 )

/* The bit of port 0 that the LPCXpresso LPC13xx LED is connected. */
#define mainLED_BIT 						( 22 )

/* The rate at which data is sent to the queue, specified in milliseconds. */
#define mainQUEUE_SEND_FREQUENCY_MS			( 200 / portTICK_RATE_MS )
/* The rate at which high is triggered to the HC-SR04 (10 micro sec.). */
#define mainTRIGGER_TIME			( 100 )

/* The number of items the queue can hold.  This is 1 as the receive task
will remove items as they are added, meaning the send task should always find
the queue empty. */
#define mainQUEUE_LENGTH					( 40 )

/* PROTOTYPES */
/*
 * This function reads a value in a queue to decode it and launch the appropriate methods.
 * Values are enqueued by sensor produced edges.
 * If there is no value on the queue, then the task enter into sleep mode until some value
 * is queued.
 *
 * @param pvParameters		A pointer to parameters data
 */
static void decoderTask( void *pvParameters );

/*
 * This function waits until a value is queued, then decode the value stored in UART3 buffer
 * If there is no value on the queue, then the task enter into sleep mode until some value
 * is queued.
 *
 * @param pvParameters		A pointer to parameters data
 */
static void midiDecoder( void *pvParameters );

/*
 * This function waits until a value is queued, then decode the value stored in ADC buffer
 * If there is no value on the queue, then the task enter into sleep mode until some value
 * is queued.
 *
 * @param pvParameters		A pointer to parameters data
 */
static void adcDecoder( void *pvParameters );

/*
 * This function waits until a value is queued, then decode the value stored in UART1 buffer
 * If there is no value on the queue, then the task enter into sleep mode until some value
 * is queued.
 *
 * @param pvParameters		A pointer to parameters data
 */
static void rnxvDecoder( void *pvParameters );

/*
 * This is a private function to retrieve the pushButton relative pin who caused the edge
 *
 * @param pin		Pin number to retreive his id
 */
uint8_t getIdByPin( unsigned long pin );


/* The queue to decode messages from interruptions edged by sensors */
static xQueueHandle decodeQueue = NULL;
/* The queue is used to listen the UART1. */
static xQueueHandle uartQueue1 = NULL;
/* The queue is used to listen the UART3. */
static xQueueHandle uartQueue3 = NULL;
/* The queue is used to do a pseudo polling in ADC inputs. */
static xQueueHandle adcQueue = NULL;

/* variables */
extern volatile uint32_t UART0Status, UART1Status, UART2Status, UART3Status;
extern volatile uint8_t UART0TxEmpty, UART1TxEmpty, UART2TxEmpty, UART3TxEmpty;
extern volatile uint8_t UART0Buffer, UART1Buffer, UART2Buffer, UART3Buffer;
extern volatile uint32_t UART0Count, UART1Count, UART2Count, UART3Count;
volatile uint32_t *push_lpc_uart_ier = &LPC_UART3->IER;
volatile uint32_t *midi_uart_count = &UART3Count;
volatile uint8_t *midi_uart_buffer = &UART3Buffer;
volatile uint32_t *rnxv_uart_count = &UART1Count;
volatile uint8_t *rnxv_uart_buffer = &UART1Buffer;
uint8_t midiPort = 3;
int midiBaurade = 115200;
/* Midi values = { Note on, Note 60, Note velocity 127, N.off, N.60, N.velocity 127,
 Control Change, Effect control 1, General Porpouse Conreoller 1 } */
uint8_t testMidi[ 9 ] = { 0x90,  0x3C, 0x7F, 0x80, 0x3C, 0x00 , 0xB0, 0x12, 0x16};
/* This is to store the push button with pin relation */
const uint8_t pushPinSel[ 7 ] = { 9, 8, 7, 6, 21, 27, 28 };
const uint8_t pushPinSelLength = sizeof( pushPinSel ) / sizeof( pushPinSel[0] );
/* This array contains the MIDI note value for each button */
uint8_t pushNotes[ 7 ] = { 0x3C, 0x3E, 0x40, 0x41, 0x43, 0x45, 0x00 };
const unsigned long operationCode[ 26 ] = { 0UL,  100UL, 200UL, 300UL, 400UL, 500UL, 600UL, 700UL, 201UL, 202UL, 203UL, 204UL, 205UL, 301UL, 302UL, 303UL, 304UL, 305UL, 206UL, 207UL, 208UL, 209UL, 306UL, 307UL, 308UL, 309UL };

/* This variable is used to prevent signal bounce */
uint8_t falling[ 8 ] = { 0, 0, 0, 0, 0, 0, 0, 0 };

volatile uint32_t ticks1; // counter for 1micro seg
volatile uint32_t ticks; // counter for 1micro seg

#ifndef _BV
#define _BV(_x_) (1UL << (_x_))
#endif

extern volatile uint32_t ADCValue[ADC_NUM];
extern volatile uint32_t BurstCounter;
extern volatile uint32_t OverRunCounter;

#if BURST_MODE
extern volatile uint32_t channel_flag;
#endif

/*-----------------------------------------------------------*/

int main(void)
{
	/* Init the semi-hosting. This is to debug only */
	printf( "\n" );

	/* Initialize buttons */
	uint8_t iter;
	for ( iter = 0; iter < pushPinSelLength; iter++){
		pushButton_startInt( 0, pushPinSel[ iter ]);
	}

	/* Init the led (P0.22) */
	led_init();
	midi_startUART( midiPort, midiBaurade );
	UARTInit( 1, midiBaurade);

	/* Initialize the pin P2.13 to trigger to HC-SR04, set value to 00 */
	/* Initialize the pin P2.11 to switch the relay when note is read */
	/* Initialize the pin P2.10 to turn on a led */
	LPC_PINCON->PINSEL4	&= ~(( 3 << 26 ) | ( 3 << 22 ) | ( 3 << 20 ));
	/* Set GPIO to out */
	LPC_GPIO2->FIODIR |= ( 1 << 13 ) | ( 1 << 11 )  | ( 1 << 10 );

	/* Initialize interruption to echo */
	LPC_GPIOINT->IO2IntEnR |= (1 << 12);
	LPC_GPIOINT->IO2IntEnF |= (1 << 12);
	/* Initialize timer to count time between edges */
	/* Power up timer */
	timer_power ( 1 ); // This timer send high to HC-SR04 each 1/10sec
	timer_power ( 2 ); // This timer is used to mesure the time of the HC-SR04 echo
	/* Set prescaler to CLCK/4 */
	timer_prescaler( 1, 0x00 );
	timer_prescaler( 2, 0x00 );
	/* set interval and activate interruption */
	timer_init( 1, ( SystemCoreClock >> 2 ) / 1000000 ); // 1 micro sec.
	timer_init( 2, ( SystemCoreClock >> 2 ) / 1000000 );

	/* Initialize ADC  */
	ADCInit( ADC_CLK );

	/* Create the queues. */
	decodeQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( unsigned long ) );
	uartQueue1 = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint8_t ) );
	uartQueue3 = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint8_t ) );
	adcQueue = xQueueCreate( mainQUEUE_LENGTH, sizeof( uint8_t ) );

	if( decodeQueue != NULL && uartQueue1 != NULL && uartQueue3 != NULL && adcQueue != NULL )
	{
		/* Start the task who launches the methods */
		xTaskCreate( decoderTask, ( signed char * ) "Deco", configMINIMAL_STACK_SIZE, NULL, mainQUEUE_RECEIVE_TASK_PRIORITY, NULL );
		/* Start the task who listen the UART input */
		xTaskCreate( midiDecoder, ( signed char * ) "midiDecoder", configMINIMAL_STACK_SIZE, NULL, mainQUEUE_RECEIVE_TASK_PRIORITY, NULL );
		/* Start the task who listen the UART input */
		xTaskCreate( adcDecoder, ( signed char * ) "adcDecoder", configMINIMAL_STACK_SIZE, NULL, mainQUEUE_RECEIVE_TASK_PRIORITY, NULL );
		/* Start the task who listen the UART input */
		//xTaskCreate( rnxvDecoder, ( signed char * ) "adcDecoder", configMINIMAL_STACK_SIZE, NULL, mainQUEUE_RECEIVE_TASK_PRIORITY, NULL );
		/* Re-Start UART to read interruptions, why? i don't know, but if don't do it two times doesn't works! */
		midi_startUART( midiPort, midiBaurade );
		/* Potentiometer */
		ADCRead( 0 );
		/* Start the tasks running. */
		vTaskStartScheduler();

	}

	/* If all is well we will never reach here as the scheduler will now be
	running.  If we do reach here then it is likely that there was insufficient
	heap available for the idle task to be created. */
	for( ;; );
}

/*-----------------------------------------------------------*/

uint8_t getIdByPin( unsigned long pin ){
	uint8_t it = 0;

	while ( it < pushPinSelLength ){
		if ( pushPinSel[ it ] == pin )
			return it;
		it++;
	}
	/* If everything it's ok, this never will be reached */
	return 0;
}

static void decoderTask( void *pvParameters )
{
	unsigned long ulReceivedValue;
	unsigned long ulReceivedValue2;
	uint8_t toSend[ 3 ] = { 0, 0, 0 };
	portTickType xNextWakeTime;
	xNextWakeTime = xTaskGetTickCount();

	for( ;; )
	{
		/* Wait until something arrives in the queue */
		xQueueReceive( decodeQueue, &ulReceivedValue, portMAX_DELAY );

		/*  Something received from the queue, it must be decoded */
		switch ( ulReceivedValue ){
			case 0: break;
			case 100UL:
				Printf_start(0, 115200);
				char str[60];
				Printf_print( 0, 0, str, (char *)"ON"/*, iter*/ );
				break;
			case 200UL: pushButton_fallingInt( 0, pushPinSel[ 0 ] ); break;
			case 201UL: pushButton_fallingInt( 0, pushPinSel[ 1 ] ); break;
			case 202UL: pushButton_fallingInt( 0, pushPinSel[ 2 ] ); break;
			case 203UL: pushButton_fallingInt( 0, pushPinSel[ 3 ] ); break;
			case 204UL: pushButton_fallingInt( 0, pushPinSel[ 4 ] ); break;
			case 205UL: pushButton_fallingInt( 0, pushPinSel[ 5 ] ); break;
			case 206UL: pushButton_fallingInt( 0, pushPinSel[ 6 ] ); break;
			case 300UL: pushButton_raiseInt( 0, pushPinSel[ 0 ] );  break;
			case 301UL: pushButton_raiseInt( 0, pushPinSel[ 1 ] );  break;
			case 302UL: pushButton_raiseInt( 0, pushPinSel[ 2 ] );  break;
			case 303UL: pushButton_raiseInt( 0, pushPinSel[ 3 ] );  break;
			case 304UL: pushButton_raiseInt( 0, pushPinSel[ 4 ] );  break;
			case 305UL: pushButton_raiseInt( 0, pushPinSel[ 5 ] );  break;
			case 306UL: pushButton_raiseInt( 0, pushPinSel[ 6 ] );  break;
			case 400UL: // PushButton Falling
				if ( xQueueReceive( decodeQueue, &ulReceivedValue2, 0 ) == pdTRUE ){
					if ( ulReceivedValue2 == pushPinSel[ 6 ]){
						ticks1 = 0;
						timer_enable( 1 );
					}
					else{ // Send Midi note
						toSend[ 0 ] = testMidi[ 0 ];
						toSend[ 1 ] = pushNotes[ getIdByPin( ulReceivedValue2 )];
						toSend[ 2 ] = testMidi[ 2 ];
						midi_sendUART( midiPort, &toSend[ 0 ] , 3 );
					}
				}
				break;
			case 500UL: // PushButton Raise
				if ( xQueueReceive( decodeQueue, &ulReceivedValue2, 0 ) == pdTRUE ){
					if ( ulReceivedValue2 == pushPinSel[ 6 ]){
						timer_disable( 1 );
					}
					else{ // Send Midi note
						toSend[ 0 ] = testMidi[ 3 ];
						toSend[ 1 ] = pushNotes[ getIdByPin( ulReceivedValue2 )];
						toSend[ 2 ] = testMidi[ 5 ];
						midi_sendUART( midiPort, &toSend[ 0 ] , 3 );
					}
				}
				break;
			case 600UL:
				break;
			case 700UL: // Echo from HC-SR04 received
				if ( xQueueReceive( decodeQueue, &ulReceivedValue2, 0 ) == pdTRUE ){
					/* Scale the value received */
					ulReceivedValue2 = ulReceivedValue2 / 4;
					if ( 0x00000070 <= ulReceivedValue2  && ulReceivedValue2 < 0x000000EF ){ // 0xEF = 0x70 + 0x7F
						/* Send Midi controller value */
						toSend[ 0 ] = testMidi[ 6 ];
						toSend[ 1 ] = testMidi[ 7 ];
						toSend[ 2 ] = ulReceivedValue2 - 0x70;
						midi_startUART( midiPort, midiBaurade );
						midi_sendUART( midiPort, &toSend[ 0 ] , 3 ); // Send Midi Control Change
					}
				}
				break;
			default: break;
		}
	}
}


static void midiDecoder( void *pvParameters )
{
	uint8_t ulReceivedValue;
	uint8_t recMidi[ 6 ];
	uint32_t result;
	for( ;; )
	{
		/* Wait until something arrives in the queue */
		xQueueReceive( uartQueue3, &ulReceivedValue, portMAX_DELAY );
		if ( *midi_uart_count >= 3 ){
		 	result = midi_readUART( 3, recMidi );
			//UARTInit( 0, midiBaurade);
			//UARTSend( 0, &recMidi[0], 1 );
			if ( result != 0 ){
				if ( recMidi[0] == testMidi[0] )
					if (recMidi[1] == testMidi[1])
						led_on();
					else if (recMidi[1] == pushNotes[1])
						/* pin to high */
						LPC_GPIO2->FIOSET = ( 1 << 11 );
					else if (recMidi[1] == pushNotes[2])
						/* pin to high */
						LPC_GPIO2->FIOSET = ( 1 << 10);
				if ( recMidi[0] == testMidi[3] )
					if (recMidi[1] == testMidi[4])
						led_off();
					else if (recMidi[1] == pushNotes[1])
						/* pin to low */
						LPC_GPIO2->FIOCLR = ( 1 << 11 );
					else if (recMidi[1] == pushNotes[2])
						/* pin to low */
						LPC_GPIO2->FIOCLR = ( 1 << 10 );
			}
		}
	}
}

static void adcDecoder( void *pvParameters )
{
	uint8_t ulReceivedValue;
	portTickType pxPreviousWakeTime;
	uint8_t toSend[ 3 ] = { 0, 0, 0 };
	uint8_t last[ 1 ] = { 0 };
	uint8_t aux;

	pxPreviousWakeTime = xTaskGetTickCount();
	for( ;; )
	{
		/* Wait until something arrives in the queue */
		xQueueReceive( adcQueue, &ulReceivedValue, portMAX_DELAY );
		aux = 0x7F - ( ADCValue[ 0 ] >> 5 );
		if ( aux != last[ 0 ] ){
			last[ 0 ] = aux;
			toSend[ 0 ] = testMidi[ 6 ];
			toSend[ 1 ] = testMidi[ 8 ];
			toSend[ 2 ] = aux;
			midi_startUART( midiPort, midiBaurade );
			midi_sendUART( midiPort, &toSend[ 0 ] , 3 ); // Send Midi Control Change
		}
		vTaskDelayUntil( &pxPreviousWakeTime,  10 / portTICK_RATE_MS ); // 1/10 sec or 100 msec
		ADCRead( 0 );

	}
}

static void rnxvDecoder( void *pvParameters )
{
	uint8_t ulReceivedValue;
	uint8_t recRnxv[ 6 ];
	uint32_t result;
	uint32_t aux;

	for( ;; )
	{
		/* Wait until something arrives in the queue */
		xQueueReceive( uartQueue1, &ulReceivedValue, portMAX_DELAY );
		if ( *rnxv_uart_count >= 3 ){
			aux = *rnxv_uart_count;
		 	result = midi_readUART( 1, recRnxv );
			UARTInit( 1, midiBaurade);
			UARTSend( 1, &recRnxv[0], aux );
		}
	}
}
/*-----------------------------------------------------------*/


void EINT3_IRQHandler (void)
{
	/* Button 0 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 0 ])) == (1 << pushPinSel[ 0 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 0 ]);
		/* Enqueue to send */
		if ( falling[0] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[3], 0 );
			falling[0] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 0 ])) == (1 << pushPinSel[ 0 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 0 ]);
		/* Enqueue to send */
		if ( falling[0] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[2], 0 );
			falling[0] = 1;
		}
		return;
	}
	/* Button 1 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 1 ])) == (1 << pushPinSel[ 1 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 1 ]);
		/* Enqueue to send */
		if ( falling[1] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[13], 0 );
			falling[1] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 1 ])) == (1 << pushPinSel[ 1 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 1 ]);
		/* Enqueue to send */
		if ( falling[1] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[8], 0 );
			falling[1] = 1;
		}
		return;
	}
	/* Button 2 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 2 ])) == (1 << pushPinSel[ 2 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 2 ]);
		/* Enqueue to send */
		if ( falling[2] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[14], 0 );
			falling[2] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 2 ])) == (1 << pushPinSel[ 2 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 2 ]);
		/* Enqueue to send */
		if ( falling[2] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[9], 0 );
			falling[2] = 1;
		}
		return;
	}
	/* Button 3 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 3 ])) == (1 << pushPinSel[ 3 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 3 ]);
		/* Enqueue to send */
		if ( falling[3] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[15], 0 );
			falling[3] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 3 ])) == (1 << pushPinSel[ 3 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 3 ]);
		/* Enqueue to send */
		if ( falling[3] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[10], 0 );
			falling[3] = 1;
		}
		return;
	}
	/* Button 4 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 4 ])) == (1 << pushPinSel[ 4 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 4 ]);
		/* Enqueue to send */
		if ( falling[4] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[16], 0 );
			falling[4] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 4 ])) == (1 << pushPinSel[ 4 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 4 ]);
		/* Enqueue to send */
		if ( falling[4] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[11], 0 );
			falling[4] = 1;
		}
		return;
	}
	/* Button 5 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 5 ])) == (1 << pushPinSel[ 5 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 5 ]);
		/* Enqueue to send */
		if ( falling[5] == 1 ) {
			xQueueSendToBack( decodeQueue, &operationCode[17], 0 );
			falling[5] = 0;
		}

		return;
	}
	else if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 5 ])) == (1 << pushPinSel[ 5 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 5 ]);
		/* Enqueue to send */
		if ( falling[5] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[12], 0 );
			falling[5] = 1;
		}
		return;
	}
	/* Button to activate HC-SR04 */
	if ((LPC_GPIOINT->IO0IntStatR & (1 << pushPinSel[ 6 ])) == (1 << pushPinSel[ 6 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 6 ]);
		/* Enqueue to send */
		if ( falling[6] == 1){
			xQueueSendToBack( decodeQueue, &operationCode[22], 0 );
			falling[6] = 0;
		}
		return;
	}
	if ((LPC_GPIOINT->IO0IntStatF & (1 << pushPinSel[ 6 ])) == (1 << pushPinSel[ 6 ]))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO0IntClr |= (1 << pushPinSel[ 6 ]);
		/* Enqueue to send */
		if ( falling[6] == 0){
			xQueueSendToBack( decodeQueue, &operationCode[18], 0 );
			falling[6] = 1;
		}
		return;
	}
	/* Echo response from HC-SR04 */
	if ((LPC_GPIOINT->IO2IntStatR & (1 << 12)) == (1 << 12))
	{
		/* Reset the Interruption */
		LPC_GPIOINT->IO2IntClr |= (1 << 12);
		if ( falling[7] == 1){
			/* Reset ticks2 */
			ticks = 0;
			/* Enable the timer */
			timer_enable( 2 );
			falling[7] = 0;
			/* Enqueue to send */
		}
		return;
	}
	if ((LPC_GPIOINT->IO2IntStatF & (1 << 12)) == (1 << 12))
	{
		unsigned long aux;
		if ( falling[7] == 0){
			aux = ticks;
			/* Reset the Interruption */
			LPC_GPIOINT->IO2IntClr |= (1 << 12);//aux = aux / 58 to obtain cm;
			/* Disable the timer */
			timer_disable( 2 );
			falling[7] = 1;
			/* Enqueue to send */
			xQueueSendToBack( decodeQueue, &operationCode[7], 0 );
			xQueueSendToBack( decodeQueue, &aux, 0 );
		}
		return;
	}
}

// This timer send high to HC-SR04 each 1/10sec
void TIMER1_IRQHandler(void)
{
	if ( ticks1 == 0 )
		LPC_GPIO2->FIOSET = ( 1 << 13 );
	else if ( ticks1 == 10 )
		LPC_GPIO2->FIOCLR = ( 1 << 13 );
	ticks1++;
	if ( ticks1 == 100000 )
		ticks1 = 0;
	/* Reset the Interruption */
	LPC_TIM1->IR = 1;
	return;
}

// This timer is used to measure the time of the HC-SR04 echo
void TIMER2_IRQHandler(void)
{
	ticks++;
	/* Reset the Interruption */
	LPC_TIM2->IR = 1;
	return;
}

// This timer is used to debounce the pushButton signal
void TIMER3_IRQHandler (void)
{
	unsigned long toSend;
	unsigned long aux;
	uint8_t toSet;

	LPC_TIM3->IR = 1;  // clear interrupt flag
	if ( pushButton_timerCallback( &toSend, &toSet) == pdTRUE )  // debounce control
	{
		aux = toSet;
		xQueueSendToFront( decodeQueue, &aux, 0 );  // enqueue to send
	  	xQueueSendToFront( decodeQueue, &toSend, 0 );  // enqueue to send
	}
	timer_disable ( 3 );
	return;
}

/*****************************************************************************
** Function name:		UART1_IRQHandler
**
** Descriptions:		UART1 interrupt handler
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void UART1_IRQHandler (void)
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;

  IIRValue = LPC_UART1->IIR;

  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_UART1->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART1Status = LSRValue;
	  Dummy = LPC_UART1->RBR;		/* Dummy read on RX to clear
								interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */
	{
	  /* If no error on RLS, normal ready, save into the data buffer. */
	  /* Note: read RBR will clear the interrupt */
	  rnxv_uart_buffer[UART1Count] = LPC_UART1->RBR;
	  UART1Count++;
	  /* Send to uartQueue1 */
	  xQueueSendToBack( uartQueue1, (uint8_t*)0x01, 0 );
	  if ( UART1Count == BUFSIZE )
	  {
		UART1Count = 0;		/* buffer overflow */
	  }
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
	/* Receive Data Available */
	rnxv_uart_buffer[UART1Count] = LPC_UART1->RBR;
	UART1Count++;
	/* Send to uartQueue1 */
	xQueueSendToBack( uartQueue1, (uint8_t*)0x01, 0 );
	if ( UART1Count == BUFSIZE )
	{
	  UART1Count = 0;		/* buffer overflow */
	}
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART1Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_UART1->LSR;		/* Check status in the LSR to see if
								valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART1TxEmpty = 1;
	}
	else
	{
	  UART1TxEmpty = 0;
	}
  }

}

/*****************************************************************************
** Function name:		UART3_IRQHandler
**
** Descriptions:		UART3 interrupt handler
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void UART3_IRQHandler (void)
{
  uint8_t IIRValue, LSRValue;
  uint8_t Dummy = Dummy;

  IIRValue = LPC_UART3->IIR;

  IIRValue >>= 1;			/* skip pending bit in IIR */
  IIRValue &= 0x07;			/* check bit 1~3, interrupt identification */
  if ( IIRValue == IIR_RLS )		/* Receive Line Status */
  {
	LSRValue = LPC_UART3->LSR;
	/* Receive Line Status */
	if ( LSRValue & (LSR_OE|LSR_PE|LSR_FE|LSR_RXFE|LSR_BI) )
	{
	  /* There are errors or break interrupt */
	  /* Read LSR will clear the interrupt */
	  UART3Status = LSRValue;
	  Dummy = LPC_UART3->RBR;		/* Dummy read on RX to clear
							interrupt, then bail out */
	  return;
	}
	if ( LSRValue & LSR_RDR )	/* Receive Data Ready */
	{
	  /* If no error on RLS, normal ready, save into the data buffer. */
	  /* Note: read RBR will clear the interrupt */
	  midi_uart_buffer[UART3Count] = LPC_UART3->RBR;
	  UART3Count++;
	  /* Send to uartQueue3 */
	  xQueueSendToBack( uartQueue3, (uint8_t*)0x01, 0 );
	  if ( UART3Count == BUFSIZE )
	  {
		UART3Count = 0;		/* buffer overflow */
	  }
	}
  }
  else if ( IIRValue == IIR_RDA )	/* Receive Data Available */
  {
	/* Receive Data Available */
	midi_uart_buffer[UART3Count] = LPC_UART3->RBR;
	UART3Count++;
	/* Send to uartQueue3 */
	xQueueSendToBack( uartQueue3, (uint8_t*)0x01, 0 );
	if ( UART3Count == BUFSIZE )
	{
	  UART3Count = 0;		/* buffer overflow */
	}
  }
  else if ( IIRValue == IIR_CTI )	/* Character timeout indicator */
  {
	/* Character Time-out indicator */
	UART3Status |= 0x100;		/* Bit 9 as the CTI error */
  }
  else if ( IIRValue == IIR_THRE )	/* THRE, transmit holding register empty */
  {
	/* THRE interrupt */
	LSRValue = LPC_UART3->LSR;		/* Check status in the LSR to see if
									valid data in U0THR or not */
	if ( LSRValue & LSR_THRE )
	{
	  UART3TxEmpty = 1;
	}
	else
	{
	  UART3TxEmpty = 0;
	}
  }
}

#if ADC_INTERRUPT_FLAG
/******************************************************************************
** Function name:		ADC_IRQHandler
**
** Descriptions:		ADC interrupt handler
**
** parameters:			None
** Returned value:		None
**
******************************************************************************/
void ADC_IRQHandler (void)
{
  uint32_t regVal;
  volatile uint32_t dummy;
  int i;

  regVal = LPC_ADC->STAT;		/* Read ADC will clear the interrupt */
  if ( regVal & 0x00003F00 )	/* check OVERRUN error first - modified, it was 0x0000FF00*/
  {
	OverRunCounter++;
  	for ( i = 0; i < ADC_NUM; i++ )
  	{
  	  regVal = (regVal & 0x00003F00) >> 0x08; /* modified, it was 0x0000FF00*/
  	  /* if overrun, just read ADDR to clear */
  	  /* regVal variable has been reused. */
      if ( regVal & _BV(i) )
  	  {
        dummy = LPC_ADC->DR[i];
  	  }
  	}
	LPC_ADC->CR &= ~((0x7<<24)|(0x1<<16));	/* stop ADC now, turn off BURST bit. */
  	//ADCIntDone = 1;
  	xQueueSendToBack( adcQueue, (uint8_t*)0x01, 0 );
  	return;
  }

  for ( i = 0; i < ADC_NUM; i++ )
  {
    if ( regVal & _BV(i) )
	{
	  ADCValue[i] = ( LPC_ADC->DR[i] >> 4 ) & 0xFFF;
	}
  }

#if BURST_MODE
  BurstCounter++;
  channel_flag |= (regVal & 0x3F); /* modified, it was 0xFF*/
  if ( (channel_flag & 0x3F) == 0x3F ) /* modified, it was 0xFF*/
  {
	/* All the bits in have been set, it indicates all the ADC
	channels have been converted. */
	LPC_ADC->CR &= ~(0x1<<16);	/* Clear BURST mode, stop ADC now */
  	channel_flag = 0;
	//ADCIntDone = 1;
  }
#else
  LPC_ADC->CR &= ~(0x7<<24);	/* stop ADC now */
  //ADCIntDone = 1;
#endif
  xQueueSendToBack( adcQueue, (uint8_t*)0x01, 0 );
  return;
}
#endif

/*-----------------------------------------------------------*/

void vApplicationIdleHook( void )
{
	/* This example does not use the idle hook to perform any processing. */
}
/*-----------------------------------------------------------*/

void vApplicationTickHook( void )
{
	/* This example does not use the tick hook to perform any processing. */
}

void vApplicationMallocFailedHook( void )
{
	/* This function will only be called if an API call to create a task, queue
	or semaphore fails because there is too little heap RAM remaining. */

	for( ;; );
}
/*-----------------------------------------------------------*/

void vApplicationStackOverflowHook( xTaskHandle *pxTask, signed char *pcTaskName )
{
	/* This function will only be called if a task overflows its stack.  Note
	that stack overflow checking does slow down the context switch
	implementation. */
	for( ;; );
}
